#define NOT_XSYSTEM 1
#define SUSPEX 1

/*
 * TEXTURE_TEST.C -- Written to test PEX 5.1+ Texture Mapping. 
 *
 *                   Designed by Bradley L. Saunders     Sep.  8, 1993
 */

/******************************************************************************/
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/******************************************************************************/

/*
 * Basic set of include files for tests
 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/PEX5/PEX.h>
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXHPlib.h>
#include <X11/PEX5/PEXExt.h>
#include <X11/PEX5/PEXExtlib.h>
#include <pexutcmap.h>          /* Color Utilities */
#include "./../txm_utility_code/param_struct.h"
#include "./../txm_utility_code/prototypes.h"

#ifndef NOT_XSYSTEM
/*
 * XSystem requirements
 */
#include <XSystemTest.h>
#include "client_utils.h"
#include "err_handler.h"

XSystemTestingControlStructure xtc;
#endif

#ifdef SUSPEX

#include <misc.h>

extern int glob_window_width;
extern int glob_window_height;
extern int glob_window_x;
extern int glob_window_y;

#endif 
char fname[1000];

void parse_error(int status)
{
  char errstr[ERRORSTRINGSIZE];

  static char *PEXTMErrorString[] =
  {
    "", /* Error values begin with 1 */
    "PEXExtBadDimension",
    "PEXExtBadDomain",
    "PEXExtBadTexelData",
    "PEXExtBadTexelDimension",
    "PEXExtBadTexelType",
    "PEXExtBadTMAlloc",
    "PEXExtBadPrimitive",
    "PEXExtBadXResource",
    "PEXExtBadAlloc",
    "PEXExtBadValue",
    "PEXExtBadTMType"
  };

  sprintf(errstr, "%s\n",PEXTMErrorString[status]);
  Error(errstr);
}

#ifdef SUSPEX
void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
{
    strcpy(test_name, "big_dang_test");
    *num_images      = 1;
    *supported_modes = IMM_MODE;

    strcpy(win_title, "Texture Mapping Test");
}

void testproc(Display *dpy, Window window, int cur_image)
{
     image_testproc(fname, "Surface Attributes", img_desc,
		      dpy,window, 0, 0, 
		      glob_window_width, glob_window_height);
}
#endif

/*
 * MAIN 
 */
main(int argc, char *argv[])
{
/*
 * PEX window stuff
 */
  Display *display;
  Window window;
  int screen;
  PEXUtVisualCriteria criteria;
  PEXUtWindowSpecification window_info;
  unsigned int unmet;
  Atom prop_atom;
  int result;
  XSizeHints hints;
  XColor returned_background;
  XVisualInfo vis_info;
  XStandardColormap cmap_info;
  PEXColorApproxEntry capx_info;
  PEXExtensionInfo *ext_info;
  char err_string[PEXErrorStringLength];
  char errstr[ERRORSTRINGSIZE];
  int values[20];

 /*
  * SUSPEX Variables
  */
  int pexmode;
  int image_depth;
  char test_name[1000]; 
  int  num_images; 
  int  supported_modes; 
  char win_title[1000];

#ifdef NOT_XSYSTEM
 /*
  * Open Display
  */
    if (!(display = XOpenDisplay(0)))
    {
      sprintf(errstr ,"Can't open %s\n",getenv("DISPLAY"));
      Error(errstr);
    }
#else
 /*
  * Init XSystem
  * I don't send the first argument because it is the name of this not the
  * actual test being run.
  */
    openXSystemTesting(argc-1, &argv[1], &xtc, 0, 0);
    display = xtc.display;

#endif

 /*
  * Get Screen info
  */
    screen = DefaultScreen(display);

 /*
  * Initialize PEXlib
  */
    if (PEXInitialize(display, &ext_info, PEXErrorStringLength, err_string))
    {
      sprintf(errstr, "PEXInitialize failed! '%s'\n", err_string);
      Error(errstr);
    }

#ifdef SUSPEX

    inquire_test_params(test_name, &num_images, &supported_modes, win_title);

    /*  Handle window geometry string if present */
    parse_window_geometry(argc, argv);

    if (parse_options2 (argc, argv, &pexmode, &image_depth, fname))
	Error("In parse_options2");

    printf("Filename: %s\n",fname);
    printf("pexmode: %d\n",pexmode);
    printf("image depth: %d\n",image_depth);

#else

 /*
  * I am getting the parameterfiel name here and adding _param to it.
  * I assume that argv[1] has the parameter filename since CATS uses
  * that same argument for archiving.
  */
  if(argc > 1)
  {
    strcpy(fname, argv[1]);
    strcat(fname, "_param");
  }
  else
    Error("No input file?");

#endif

 /*
  * Set window size and position hints
  */
    hints.x = glob_window_x;
    hints.y = glob_window_y;
    hints.width = glob_window_width;
    hints.height = glob_window_height;
    hints.flags = (USSize|USPosition);

 /*
  * Create Window and Cmap
  */
    criteria.hard_criteria_mask = (PEXUtMinColors                 | 0);
    criteria.soft_criteria_mask = (PEXUtStandardColormapProperty  | 0);
    criteria.min_colors = 2;
    criteria.standard_colormap_property = True;

    window_info.attr_mask = CWEventMask;
    window_info.attrs.event_mask = (ExposureMask         |
                                    VisibilityChangeMask |
                                    StructureNotifyMask  | 0);

    window_info.title = "Texture Map Test Window";
    window_info.size_hints = hints;
    window_info.parent = RootWindow (display, screen);
    window_info.border_width = 0;
    window_info.background_color_name = "black";
    window_info.border_color_name = "white";
  
    result = PEXUtCreateWindowAndColormap (display,
                                           screen,
                                           &criteria,
                                           &window_info,
                                           &window,
                                           &vis_info,
                                           &cmap_info,
                                           &capx_info,
                                           &unmet,
                                           &prop_atom,
                                           &returned_background);

    if (result != PEXUtSuccess)
    {
      static char *PEXUtFailStrings[] =
      {
        "PEXUtAllocFailure",          /* -4 maps to 0 */
        "PEXUtPEXFailure",            /* -3 maps to 1 */
        "PEXUtXFailure",              /* -2 maps to 2 */
        "PEXUtCriteriaFailure",       /* -1 maps to 3 */
        "PEXUtSuccess",               /*  0 maps to 4 */
        "PEXUtQualifiedSuccess",      /* +1 maps to 5 */
      };

      sprintf(errstr,"PEXUtCreateWindowAndColormap() fails: %s\n",
              PEXUtFailStrings[ result + 4 ]);
      Error(errstr);
    }

#ifdef SUSPEX
    setup_LUT(display, window, &capx_info);
#endif 

#ifndef NOT_XSYSTEM
    /*
     * Use this utility so that we only archive the windows area.
     */
    values[0] = WINDOW_SMART_EXTENT;
    values[1] = window;
    defineXSystemTestControlParm(&xtc, ARCHIVE_EXTENT, values);

    /*
     * Use this utilitiy so that we only archive the window HW
     */
    values[0] = WINDOW_SMART_HW_CONFIG;
    values[1] = window;
    values[2] = -1;
    defineXSystemTestControlParm (&xtc, ARCHIVE_MODE, values);
#endif

 /**********************************************************************
  * Test begins here
  *********************************************************************/
{
 /*
  * Create some structures to work with
  */
    Input_Parameters in;
    int	status;
    PEXRenderer renderer;

 /*
  * Implementation dependent constants 
  */
    PEXImpDepConstant *constants;
    unsigned short const_names[4];
    unsigned long num_const = 4;
    int i, j, r;
    static PEXStructure struct_id;
    unsigned long mask[2];
    PEXExtRendererAttributes *attr;
    PEXTableIndex lights_on[6];
    int num_lights;

 /*
  * Filtered TM 
  */
    unsigned power_of_two_req;
    unsigned square_req;
    Window rgbwin;
    Window alphawin;

 /* 
  * Create TM 
  */
    PEXExtTextureMap *texture_ids;

 /* 
  * TM Description 
  */
    PEXExtTMDescription *texture_map;

 /*
  * Parse the Parameter File
  */
      Parse_File(fname, &in);

 /*
  * Allocate memory for the texture ids and descriptions
  */
    texture_ids = (PEXExtTextureMap *)malloc(in.TEXTURES * 
                  sizeof(PEXExtTextureMap));
    texture_map = (PEXExtTMDescription *)malloc(in.TEXTURES * 
                  sizeof(PEXExtTMDescription));
 /*
  * Get implementation dependent constants 
  */
    const_names[0] = PEXExtIDMaxTextureMaps;
    const_names[1] = PEXExtIDMaxFastTMSize;
    const_names[2] = PEXExtIDPowerOfTwoTMSizesRequired;
    const_names[3] = PEXExtIDSquareTMRequired;

    if (PEXGetImpDepConstants (display, 
                               window, 
                               num_const, 
                               const_names,
                               &constants) == 0)
    {
      Error("Unable to inquire Implementation Dependent constants");
    }

 /*
  * Create Filtered TM 
  */
    power_of_two_req = constants[2].integer;
    square_req = constants[3].integer; 

    for(i=0; i<in.TEXTURES; i++)
    {
      if(in.call_create_filtered_tm[i] == True)
      {
	status = PEXExtCreateFilteredTM(in.domain[i], 
                                        &in.domain_data[i],
                                        power_of_two_req,
	                                square_req, 
                                        &in.base_map[i],
	                                &in.texel_array[i]);
        if(status != 0)
	{
	  fprintf (stderr, "CreateFilteredTM:\n");
          parse_error(status);
	}
      } /* End if call Create Filtered Texture Map true */

   /*
    * Create Filtered TM From Window
    */
      if(in.call_create_filtered_tm_from_window[i] == True)
      {
        Put_Texture_In_Window(display, 
                              &in, 
                              i, 
                              in.window_depth[i], 
                              &rgbwin, 
                              &alphawin);

        status = PEXExtCreateFilteredTMFromWindow(display,
                                                  in.domain[i], 
                                                  &in.domain_data[i],
                                                  power_of_two_req,
                                                  square_req, 
                                                  in.luminance_selector[i],
                                                  rgbwin,
                                                  in.alpha_selector[i],
                                                  alphawin,
	                                          &in.texel_array[i]);

        XDestroyWindow(display, rgbwin);
        if(alphawin)
          XDestroyWindow(display, alphawin);

        if(status != 0)
        {
          fprintf (stderr, "CreateFilteredTMFromWindow:\n");
          parse_error(status);
        }
      } /* End if call Filtered Texture Map From Window true */

   /*
    * Create a TM 
    */
    if(in.call_create_filtered_tm_from_window[i] == False &&
       in.call_create_filtered_tm[i] == False)
      in.texel_array[i] = &in.base_map[i];

    if(in.Do_CreateTM == True)
    {
      texture_ids[i] = PEXExtCreateTM(display,
                                      in.domain[i],
                                      &in.domain_data[i],
                                      in.texel_array[i]);

   /*
    * Create a TM Description
    */
      texture_map[i] = PEXExtCreateTMDescription(display,
                                                 in.parameterization[i],
                                                 &in.param_data[i],
                                                 in.rendering_order[i],
                                                 1,
                                                 &texture_ids[i]);
  
   /*
    * Free the filtered TM 
    */
      free(in.base_map[i].array.rgb);

      if(in.call_create_filtered_tm_from_window[i] == True ||
         in.call_create_filtered_tm[i] == True)
      {
        PEXExtFreeFilteredTM (in.domain[i],
                              &in.domain_data[i],
                              in.texel_array[i]);
      }
    } /* End if Do_CreateTM */
  } /* END for i for all textures */

 /*
  * Fill out structures with texture coord 
  */
  if(in.Do_Call_Coord_Utility == True)
  {
    for(i=0; i<in.obj; i++)
    {
      switch(in.pex_coord_type[i])
      {
      case FASWD:
        for(j=0; j<in.POLYS[i]; j++)
        {
          status =
            PEXExtTMCoordFillAreaSetWithData(&in.coord_data, 
                                             in.vertex_fp_data_size,
                                             in.facet_att[i], 
                                             in.vertex_att[i], 
                                             in.color_type[i],
                                             1, 
                                             &in.facet_data[j],
                                             &in.vertex_list[i][j]);
          if(status != 0)
          {
            fprintf(stderr,"PEXExtTMCoordFillAreaSetWithData:\n");
            parse_error(status);
          }
        }
      break;
   
      case SOFAS:
        status = PEXExtTMCoordSetOfFillAreaSets(&in.coord_data,
                                                in.vertex_fp_data_size,
                                                in.facet_att[i],
                                                in.vertex_att[i],
                                                in.color_type[i],
                                                in.count[i],
                                                in.facet_array,
                                                in.POINTS[i],
                                                in.vertex_array[i],
                                                in.index_count[i],
                                                in.connectivity[i]);
        if(status != 0)
        {
          fprintf(stderr,"PEXExtTMCoordSetOfFillAreaSets:\n");
          parse_error(status);
        }
      break;
   
      case TS:
        status = PEXExtTMCoordTriangleStrip(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[i],
                                            in.vertex_att[i],
                                            in.color_type[i],
                                            in.facet_array,
                                            in.POINTS[i]-2,
                                            in.vertex_array[i]);
        if(status != 0)
        {
          fprintf(stderr,"PEXExtTMCoordTriangleStrip:\n");
          parse_error(status);
        }
      break;
   
      case QM:
        status = PEXExtTMCoordQuadrilateralMesh(&in.coord_data,
                                                in.vertex_fp_data_size,
                                                in.facet_att[i],
                                                in.vertex_att[i],
                                                in.color_type[i],
                                                in.facet_array,
                                                in.height[i],
                                                in.width[i],
                                                in.vertex_array[i]);
        if(status != 0)
        {
          fprintf(stderr,"PEXExtTMCoordQuadrilateralMesh:\n");
          parse_error(status);
        }
      break;
      } /* End switch */
    } /* End for i = number of objs */
  } /* End if Do_Call_Coord_Utility */

 /*
  * Create a Renderer
  */
    renderer = PEXCreateRenderer (display, 
                                  window, 
                                  (unsigned long)NULL,
                                  (PEXRendererAttributes *) attr);
    if (renderer == None)
    {
      sprintf(errstr,"Could not create a Renderer\n");
      Error(errstr);
    }

 /*
  * Fill the LUTS
  */
    attr = (PEXExtRendererAttributes *)
      malloc(sizeof(PEXExtRendererAttributes));
    mask[0] = mask[1] = 0L;

 /* 
  * View LUT 
  */
  if(in.Do_Use_Custom_View_Point == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXRAViewTable); 
    attr->view_table = PEXCreateLookupTable(display, window, PEXLUTView);
    set_view(display, attr->view_table, in);
  } /* End if Do_Use_Custom_View_Point */


 /* 
  * Light LUT 
  */
  if(in.Do_Use_Custom_Lights == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXRALightTable); 
    attr->light_table = PEXCreateLookupTable(display, window, PEXLUTLight);
    set_lights(display, attr->light_table, in, &num_lights);
    for(i=0; i<num_lights; i++)
      lights_on[i] = (PEXTableIndex)(i + 1);
  } /* End if Do_Use_Custom_Lights */

 

 /* 
  * Depth LUT 
  */
  if(in.Do_Use_Depth_Cue == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXRADepthCueTable); 
    attr->depth_cue_table = 
      PEXCreateLookupTable(display, window, PEXLUTDepthCue);
    set_depth_cue(display, attr->depth_cue_table, in);
  } /* End if Do_Use_Depth_Cue */

  if(in.Do_Use_Coord_Source)
  {
    PEXExtSetRendererAttributeMask(mask, PEXExtRATMCoordSourceTable); 
    attr->tm_coord_source_table =
      PEXCreateLookupTable(display, 
                           window, 
                           PEXExtLUTTMCoordSource);
    for(i=0; i<in.num_coordsource; i++)
    {
      PEXExtSetTableEntries(display, 
                            attr->tm_coord_source_table,
                            in.coordsource_indx[i], 
                            1, 
                            PEXExtLUTTMCoordSource,
			    &in.coordsource[i]);
    }
  } /* End if Do_Use_Coord_Source */

  if(in.Do_Use_Composition == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXExtRATMCompositionTable); 
    attr->tm_composition_table =
      PEXCreateLookupTable(display, 
                           window, 
                           PEXExtLUTTMComposition);
    for(i=0; i<in.num_composition; i++)
    {
      PEXExtSetTableEntries(display, 
                            attr->tm_composition_table,
                            in.composition_indx[i], 
                            1, 
                            PEXExtLUTTMComposition,
			    &in.composition[i]);
    }
  } /* End if Do_Use_Composition */

  if(in.Do_Use_Sampling_LUT == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXExtRATMSamplingTable); 
    attr->tm_sampling_table =
      PEXCreateLookupTable(display, 
                           window, 
                           PEXExtLUTTMSampling);
    for(i=0; i<in.num_sampling; i++)
    {
      PEXExtSetTableEntries(display, 
                            attr->tm_sampling_table,
                            in.sampling_indx[i], 
                            1, 
                            PEXExtLUTTMSampling,
		            &in.sampling[i]);
    }
  } /* End if Do_Use_Sampling_LUT */
 
  if(in.Do_Use_Binding_LUT == True)
  {
    PEXExtSetRendererAttributeMask(mask, PEXExtRATMBindingTable); 
    attr->tm_binding_table =
      PEXCreateLookupTable(display, 
                           window, 
                           PEXExtLUTTMBinding);
    for(i=0; i<in.num_binding; i++)
    {
      in.binding[i].tm_description_id = texture_map[in.binding_info[i]];
      PEXExtSetTableEntries(display, 
                            attr->tm_binding_table,
                            in.binding_indx[i], 
                            1,
                            PEXExtLUTTMBinding,
                            &in.binding[i]);
    }
  } /* End if Do_Use_Binding_LUT */

 /*
  * Set Custom Background Color
  */
    if(in.Do_Custom_Background_Color == True)
    {
      attr->clear_image = True;
      attr->background_color.type = in.bkgd_color.type;
      attr->background_color.value.rgb.red = in.bkgd_color.value.rgb.red;
      attr->background_color.value.rgb.green = in.bkgd_color.value.rgb.green;
      attr->background_color.value.rgb.blue = in.bkgd_color.value.rgb.blue;
      PEXExtSetRendererAttributeMask(mask, PEXRAClearImage); 
      PEXExtSetRendererAttributeMask(mask, PEXRABackgroundColor); 
    }
    else 
    {
      attr->clear_image = True;
      attr->background_color.type = PEXColorTypeRGB;
      attr->background_color.value.rgb.red = 0.0;
      attr->background_color.value.rgb.green = 0.0;
      attr->background_color.value.rgb.blue = 0.0;
      PEXExtSetRendererAttributeMask(mask, PEXRAClearImage); 
      PEXExtSetRendererAttributeMask(mask, PEXRABackgroundColor); 
    }

 /*
  * Turn on Hidden Surface
  */

  attr->hlhsr_mode = PEXHLHSRZBuffer;
  PEXExtSetRendererAttributeMask(mask, PEXRAHLHSRMode);
     
 /*
  * Turn on texture mapping
  */

 /*
  * Change the ExtRenderer
  */
    PEXExtChangeRenderer (display, renderer, mask, attr);

 /*
  * Set up rendering structure
  */

    if(pexmode & IMM_MODE) /* PEXOCRender */
    {
      in.render_target = (XID)renderer;
      in.req_type = PEXOCRender;
   /*
    * Begin Rendering 
    */
      PEXBeginRendering(display, window, renderer);
    }
    else if( pexmode & STR_MODE )
    {
      in.render_target = (XID)PEXCreateStructure(display);
      in.req_type = PEXOCStore;
    }
    else 
    {
	printf("Invalid mode -- only IMM and STR modes supported \n");
	exit(0);
    }
 /*
  * Set Reflection Model
  */
    PEXSetReflectionModel(display,
                          in.render_target,
                          in.req_type,
                          in.reflec_model);

    PEXSetBFReflectionModel(display,
                            in.render_target,
                            in.req_type,
                            in.BFreflec_model);

    if(in.BFreflec_model != PEXInteriorStyleEmpty)
      PEXSetFacetDistinguishFlag(display, in.render_target, in.req_type, True);


    if(in.Do_Use_Custom_View_Point == True)
      PEXSetViewIndex(display, in.render_target, in.req_type, 1);

   /* 
    * Have PEX compute all lighting effects. 
    */
    if(in.Do_Use_Custom_Lights == True)
    {
      PEXSetLightSourceState(display, 
                             in.render_target, 
                             in.req_type, 
                             num_lights, 
                             lights_on,
                             0, 
                             (PEXTableIndex*)NULL);
    } /* End if Do_Use_Custom_Lights */

    if(in.Do_Use_Depth_Cue == True)
      PEXSetDepthCueIndex(display, in.render_target, in.req_type, 1);
    
 /*
  * Set interior style
  */
    PEXSetInteriorStyle(display, 
                        in.render_target, 
                        in.req_type, 
                        in.style);

 /*
  * Set BF interior style
  */
    PEXSetBFInteriorStyle(display, 
                          in.render_target, 
                          in.req_type, 
                          in.BFstyle);

 /* 
  * Set Culling 
  */ 
    PEXSetFacetCullingMode(display, 
                           in.render_target, 
                           in.req_type, 
                           in.culling_mode);

 /*
  * Set Surface Interpolation Method
  */
    if((in.vertex_att && PEXGAColor) == PEXGAColor)
      PEXSetSurfaceInterpMethod(display, 
                                in.render_target,
                                in.req_type, 
                                PEXSurfaceInterpColor);
 
 /*
  * Set Perspective Correction
  */
  if(in.Do_SetTMPerspectiveCorrection == True)
    PEXExtSetTMPerspectiveCorrection(display,
                                     in.render_target,
                                     in.req_type,
                                     in.method);

 /*
  * Set Sample Frequency
  */
  if(in.Do_SetTMSampleFrequency == True)
    PEXExtSetTMSampleFrequency(display,
                               in.render_target,
                               in.req_type,
                               in.frequency);

 /*
  * Resource Hints
  */
  if(in.Do_SetTMResourceHints == True)
    PEXExtSetTMResourceHints(display, 
                             in.render_target, 
                             in.req_type, 
                             in.opt_hint, 
                             in.opt_count, 
                             in.priorities);

 /*
  * Active TMs 
  */
    PEXExtSetActiveTextures(display, 
                            in.render_target, 
                            in.req_type, 
                            in.number_FF_textures, 
                            in.FF_texture_indices);

    PEXExtSetBFActiveTextures(display,
                              in.render_target, 
                              in.req_type, 
                              in.number_BF_textures,
                              in.BF_texture_indices);

 /*
  * Extended Primitives
  */
  for(i=0; i<in.obj; i++)
  {
    switch(in.pex_coord_type[i])
    {
    case FASWD:
      for(j=0; j<in.POLYS[i]; j++)
      {
        PEXExtFillAreaSetWithData(display,
                                  in.render_target,
                                  in.req_type,
                                  in.shape_hint,
                                  in.ignore_edges,
                                  in.contour_hint,
                                  in.vertex_fp_data_size,
                                  in.facet_att[i],
                                  in.vertex_att[i],
                                  in.color_type[i],
                                  1,
                                  &in.facet_data[j],
                                  &in.vertex_list[i][j]);
      }
    break;

    case SOFAS:
      PEXExtSetOfFillAreaSets(display,
                              in.render_target,
                              in.req_type,
                              in.shape_hint,
                              in.vertex_fp_data_size,
                              in.facet_att[i],
                              in.vertex_att[i],
                              in.edge_flag,
                              in.contour_hint,
                              False,
                              in.color_type[i],
                              in.count[i],
                              in.facet_array,
                              in.POINTS[i],
                              in.vertex_array[i],
                              in.index_count[i],
                              (PEXSwitch *)NULL,
                              in.connectivity[i]);
    break;

    case TS:
      PEXExtTriangleStrip(display,
                          in.render_target,
                          in.req_type,
                          in.vertex_fp_data_size,
                          in.facet_att[i],
                          in.vertex_att[i],
                          in.color_type[i],
                          in.facet_array,
                          in.POINTS[i]-2,
                          in.vertex_array[i]);
    break;

    case QM:
      PEXExtQuadrilateralMesh(display,
                              in.render_target,
                              in.req_type,
                              in.shape_hint,
                              in.vertex_fp_data_size,
                              in.facet_att[i],
                              in.vertex_att[i],
                              in.color_type[i],
                              in.facet_array,
                              in.height[i],
                              in.width[i],
                              in.vertex_array[i]);
    break;
    }
  } /* End for all objs */

 /*
  * If store then render the structure
  */
    if(r == 1) /* PEXOCStore */
    {
#ifndef NOT_XSYSTEM
      values[0] = 1;
      defineXSystemTestControlParm(&xtc, RESET_FRAME_NUMBER, values);
#endif

      XClearWindow(display, window);
      PEXRenderNetwork(display, window, renderer, in.render_target);
      XFlush(display);
    }

    if(r == 0) /* PEXOCRender */
    {
   /*
    * End Rendering 
    */
      PEXEndRendering(display, renderer, 0);
    }
 /*
  * Do a check on the test image.
  * Commented out for now because only stderr needs to be checked.
  */

    XSync(display, 0);

#ifndef NOT_XSYSTEM
    XSystemCheck("", &xtc);
#endif
 
 /*
  * Free the TMs 
  */
    if(in.Do_CreateTM == True)
      for(i=0; i<in.TEXTURES; i++)
      {
        PEXExtFreeTMDescription (display, texture_map[i]);
        PEXExtFreeTM (display, texture_ids[i]);
      }
} /* Test ends here */

 /**********************************************************************
  * Test ends here
  *********************************************************************/

#ifdef SUSPEX
    testproc(display, window, 1);
#endif

#ifdef BBA
    _bA_dump();
#endif 

#ifdef NOT_XSYSTEM
 /*
  * Close display
  */
#ifndef SUSPEX
{
  char ch = getchar();
}
#endif

    XCloseDisplay(display);
#else
 /*
  * Terminate the testing system.
  */
    closeXSystemTesting(&xtc);
#endif
}

#ifdef SUSPEX
extern char *optarg;            /* for getopt */
extern int optind, opterr;

int parse_options2 (int argc, char **argv, int *pexmode, int *image_depth, 
    char *fname)
{
    int			c;
    char		dummy[2];

#ifndef XSYSTEM
    /* process command line options */
    while ((c = getopt(argc,argv,"d:inpsvwf:")) != EOF) {
        switch(c) {
            case 'd':
                *image_depth = atoi(optarg);
                break;
            case 'i':
                *pexmode |= IMM_MODE;
                break;
            case 'n':
                *pexmode |= NON_OC_MODE;
                break;
            case 'p':
                *pexmode |= WKS_MODE;
                break;
            case 's':
                *pexmode |= STR_MODE;
                break;
            case 'v':
                glob_verbose_mode = 1;
                break;
            case 'f':
                if(fname != NULL) {
		    strcpy(fname,optarg );
		}
                break;
	    case 'w':	/* wait for messing around with debugger :-) */
	        printf("Test waiting for <CR>: ");
	        scanf("%c", dummy);
		printf("-continuing\n");
	        break;
	    case '?':
	        printf("usage: %s: -[ipsvw]\n",argv[0]);
	        printf("-i		immediate mode\n");
	        printf("-n		non-oc mode\n");
	        printf("-p		PHIGS workstation mode\n");
	        printf("-s		structure rendering mode\n");
	        printf("-v		verbose mode (test descriptions)\n");
	        printf("-w		wait for <CR> before continuing\n");
	        printf("default	immediate mode\n");
	        return(1);
	}
    }
#endif

    return 0;
}
#endif
