/*
******************************************************************************
** PEXlib Texture Mapping Test Code 
**
** $Source:$
** $Revision:$
** $Date:$
** $Author:$
******************************************************************************
*/

#ifdef	TXM_DEBUG

#define HPPEX_TXM_DBG_SERV_STATE        0x00000001
#define HPPEX_TXM_DBG_DIDD_STATE        0x00000002
#define HPPEX_TXM_DBG_DEALY_STATE       0x00000004

extern unsigned int _hp_txm_debug_flags;

#endif

/*
 *  THIS IS A PRELIMINARY VERSION!!
 */

/*#define NOT_XSYSTEM 1*/
/*#define SUSPEX 1*/

/*
 * TEXTURE_TEST.C -- Written to test PEX 5.1+ Texture Mapping. 
 *
 *                   Designed by Bradley L. Saunders     Sep.  8, 1993
 */

/******************************************************************************/
/*                                                                            */
/*  (c) Copyright Hewlett-Packard Company, 1993, Fort Collins, Colorado       */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted,          */
/*  provided that the above copyright notices appear in all copies and that   */
/*  both the copyright notices and this permission notice appear in           */
/*  supporting documentation, and that the name of Hewlett-Packard not be     */
/*  used in advertising or publicity pertaining to distribution of the        */
/*  software without specific, written prior permission.                      */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/******************************************************************************/

/*
 * Basic set of include files for tests
 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/PEX5/PEX.h>
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXHPlib.h>
#include <X11/PEX5/PEXExt.h>
#include <X11/PEX5/PEXExtlib.h>
#include <pexutcmap.h>          /* Color Utilities */
#include "./../txm_utility_code/param_struct.h"
#include "./../txm_utility_code/prototypes.h"
#include "err_handler.h"

#ifndef NOT_XSYSTEM
/*
 * XSystem requirements
 */
#include <XSystemTest.h>
#include "client_utils.h"
#include "err_handler.h"

XSystemTestingControlStructure xtc;
#endif

#ifdef SUSPEX

#include <misc.h>

#endif 
#define HPPEX_PROCEDURES

void parse_error(int status)
{
  char errstr[ERRORSTRINGSIZE];

  switch(status)
  {
  case 0:
    sprintf(errstr, "No Error was returned\n");
  break;

  case PEXExtBadDimension:
    sprintf(errstr, "PEXExtBadDimension\n");
  break;

  case PEXExtBadDomain:
    sprintf(errstr, "PEXExtBadDomain\n");
  break;

  case PEXExtBadTexelData:
    sprintf(errstr, "PEXExtBadTexelData\n");
  break;

  case PEXExtBadTexelDimension:
    sprintf(errstr, "PEXExtBadTexelDimension\n");
  break;

  case PEXExtBadTexelType:
    sprintf(errstr, "PEXExtBadTexelType\n");
  break;

  case PEXExtBadTMAlloc:
    sprintf(errstr, "PEXExtBadTMAlloc");
  break;

  case PEXExtBadPrimitive:
    sprintf(errstr, "PEXExtBadPrimitive\n");
  break;

  case PEXExtBadXResource:
    sprintf(errstr, "PEXExtBadXResource\n");
  break;

  case PEXExtBadAlloc:
    sprintf(errstr, "PEXExtBadAlloc\n");
  break;

  case PEXExtBadValue:
    sprintf(errstr, "PEXExtBadValue\n");
  break;

  case PEXExtBadTMType:
    sprintf(errstr, "PEXExtBadTMType\n");
  break;
  }
  fprintf(stderr, "%s",errstr);
}

#ifdef SUSPEX
void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
{
    strcpy(test_name, "big_dang_test");
    *num_images      = 1;
    *supported_modes = IMM_MODE;

    strcpy(win_title, "Texture Mapping Test");
}

void testproc(Display *dpy, Window window, int cur_image)
{
     image_testproc("surf_attrs", "Surface Attributes", img_desc,
		      dpy,window, 0, 0, 
		      glob_window_width, glob_window_height);
}
#endif

/*
 * MAIN 
 */
main(int argc, char *argv[])
{
  int values[20];

/*
 * PEX window stuff
 */
  Display *display;
  Window window;
  int screen;
  PEXUtVisualCriteria criteria;
  PEXUtWindowSpecification window_info;
  unsigned int unmet;
  Atom prop_atom;
  int result;
  XSizeHints hints;
  XColor returned_background;
  XVisualInfo vis_info;
  XStandardColormap cmap_info;
  PEXColorApproxEntry capx_info;
  PEXExtensionInfo *ext_info;
  char err_string[PEXErrorStringLength];
  char fname[1000];
  int pexmode;
  int image_depth;
  char test_name[1000]; 
  int  num_images; 
  int  supported_modes; 
  char win_title[1000];


  /* Not always needed, available if so */
  esEscapeDblBuffer dblBuffer;
  esEscapeSwapBuffer swapBuffer;

 /*
  * Test specific data
  */
    char errstr[ERRORSTRINGSIZE];
    PEXRenderer renderer;
    PEXRendererAttributes set_attrs;

    char *param_string;

/* JMF */
#ifdef	TXM_DEBUG
_hp_txm_debug_flags = HPPEX_TXM_DBG_DIDD_STATE |
			HPPEX_TXM_DBG_DEALY_STATE; 
#endif

#ifdef NOT_XSYSTEM
 /*
  * Open Display
  */
    if (!(display = XOpenDisplay(0)))
    {
      sprintf(errstr ,"Can't open %s\n",getenv("DISPLAY"));
      Error(errstr);
    }
#else
    openXSystemTesting(argc, &argv[0], &xtc, 0, 0);
    display = xtc.display;

#endif

 /*
  * Set up error handler
    _hppex_test_init_error_handler();
  */


 /*
  * Get Screen info
  */
    screen = DefaultScreen(display);

 /*
  * Initialize PEXlib
  */
    if (PEXInitialize(display, &ext_info, PEXErrorStringLength, err_string))
    {
      sprintf(errstr, "PEXInitialize failed!\n");
      Error(errstr);
    }

#ifdef SUSPEX

    inquire_test_params(test_name, &num_images, &supported_modes, win_title);

    /*  Handle window geometry string if present */
    parse_window_geometry(argc, argv);

    if (parse_options2 (argc, argv, &pexmode, &image_depth, fname))
	Error("In parse_options2");

    printf("Filename: %s\n",fname);
    printf("pexmode: %d\n",pexmode);
    printf("image depth: %d\n",image_depth);

#else

 /*
  * I am getting the parameterfiel name here and adding _param to it.
  * I assume that argv[0] has the parameter filename since CATS uses
  * that same argument for archiving.
  */
  strcpy(fname, argv[0]);
  strcat(fname, "_param");

#endif

 /*
  * Set window size and position hints
  */
    hints.x = 300;
    hints.y = 60;
    hints.width = 600;
    hints.height = 600;
    hints.flags = (USSize|USPosition);

 /*
  * Create Window and Cmap
  */
    criteria.hard_criteria_mask = (PEXUtMinColors                 | 0);
    criteria.soft_criteria_mask = (PEXUtStandardColormapProperty  | 0);
    criteria.min_colors = 2;
    criteria.standard_colormap_property = True;

    window_info.attr_mask = CWEventMask;
    window_info.attrs.event_mask = (ExposureMask         |
                                    VisibilityChangeMask |
                                    StructureNotifyMask  | 0);

    window_info.title = "Texture Map Test Window";
    window_info.size_hints = hints;
    window_info.parent = RootWindow (display, screen);
    window_info.border_width = 0;
    window_info.background_color_name = "black";
    window_info.border_color_name = "white";
  
    result = PEXUtCreateWindowAndColormap (display,
                                           screen,
                                           &criteria,
                                           &window_info,
                                           &window,
                                           &vis_info,
                                           &cmap_info,
                                           &capx_info,
                                           &unmet,
                                           &prop_atom,
                                           &returned_background);

    if (result != PEXUtSuccess)
    {
      static char *PEXUtFailStrings[] =
      {
        "PEXUtAllocFailure",          /* -4 maps to 0 */
        "PEXUtPEXFailure",            /* -3 maps to 1 */
        "PEXUtXFailure",              /* -2 maps to 2 */
        "PEXUtCriteriaFailure",       /* -1 maps to 3 */
        "PEXUtSuccess",               /*  0 maps to 4 */
        "PEXUtQualifiedSuccess",      /* +1 maps to 5 */
      };

      sprintf(errstr,"PEXUtCreateWindowAndColormap() fails: %s\n",
              PEXUtFailStrings[ result + 4 ]);
      Error(errstr);
    }

#ifdef SUSPEX
    setup_LUT(display, window, &capx_info);
#endif 

#ifndef NOT_XSYSTEM
 /* 
  * Use this utility so that we only archive the windows area. 
    values[0] = WINDOW_SMART_EXTENT;
    values[1] = window;
    defineXSystemTestControlParm(&xtc, ARCHIVE_EXTENT, values);
  */


 /*
  * Use this utilitiy so that we only archive the window HW
    values[0] = WINDOW_SMART_HW_CONFIG;
    values[1] = window;
    values[2] = -1;
    defineXSystemTestControlParm (&xtc, ARCHIVE_MODE, values);
  */
#endif

 /**********************************************************************
  * Test begins here
  *********************************************************************/
{
 /*
  * Create some structures to work with
  */
    Input_Parameters in;
    int	status;
    unsigned long err_mask;
    int i;
    PEXEnumTypeIndex enum_sv;
    int int_sv;
    int obj=0;

 /*
  * Fill up a polygon for FASWD that has all 0's.
  * The polygon coming from big_dang_test will be used for
  * SOFAS, TS, QM.
  */
    /* Four vertices and space for vertex and 5 for with data*/
    float verts[4][8];
    PEXExtListOfVertex areas[1];

    /* Build the background fill area. */
    verts[0][0] = 0; verts[0][1] = 0; verts[0][2] = 0;
    verts[1][0] = 0; verts[1][1] = 0; verts[1][2] = 0;
    verts[2][0] = 0; verts[2][1] = 0; verts[2][2] = 0;
    verts[3][0] = 0; verts[3][1] = 0; verts[3][2] = 0;
    areas[0].count = 4;
    areas[0].vertices.no_data = (PEXCoord *)verts[0];


 /*
  * Set up the Error mask
  */
    err_mask = ERROR_CODE;

 /*
  * Parse the Parameter File
  */
      Parse_File(fname, &in);

 /*
  * Call all TMCoord* with invalid projection type 
  */
    fprintf(stderr, "Invalid Projection Type\n");

    i = 0;
    enum_sv = in.coord_data.projection;
    in.coord_data.projection = (PEXEnumTypeIndex)0;

    status =
      PEXExtTMCoordFillAreaSetWithData(&in.coord_data, 
                                       in.vertex_fp_data_size,
                                       in.facet_att[obj], 
                                       in.vertex_att[obj], 
                                       in.color_type[obj],
                                       1, 
                                       in.facet_data, 
                                       areas);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordFillAreaSetWithData - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }
 
    status = PEXExtTMCoordSetOfFillAreaSets(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.count[obj],
                                            in.facet_array,
                                            in.POINTS[obj],
                                            in.vertex_array[obj],
                                            in.index_count[obj],
                                            in.connectivity[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordSetOfFillAreaSets - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }

    status = PEXExtTMCoordTriangleStrip(&in.coord_data,
                                        in.vertex_fp_data_size,
                                        in.facet_att[obj],
                                        in.vertex_att[obj],
                                        in.color_type[obj],
                                        in.facet_array,
                                        in.POINTS[obj]-2,
                                        in.vertex_array[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordTriangleStrip - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }

    status = PEXExtTMCoordQuadrilateralMesh(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.facet_array,
                                            in.height[obj],
                                            in.width[obj],
                                            in.vertex_array[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordQuadrilateralMesh - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }
    in.coord_data.projection = enum_sv; /* Reset to a good projection type */

 /*
  * Call TMCoord* with invalid texture coord location
  */
    fprintf(stderr, "Invalid Texture Coord Location\n");

    int_sv = in.coord_data.fp_data_index;
    in.coord_data.fp_data_index = 100; 
    status =
      PEXExtTMCoordFillAreaSetWithData(&in.coord_data, 
                                       in.vertex_fp_data_size,
                                       in.facet_att[obj], 
                                       in.vertex_att[obj], 
                                       in.color_type[obj],
                                       1, 
                                       in.facet_data, 
                                       areas);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordFillAreaSetWithData - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }
 
    status = PEXExtTMCoordSetOfFillAreaSets(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.count[obj],
                                            in.facet_array,
                                            in.POINTS[obj],
                                            in.vertex_array[obj],
                                            in.index_count[obj],
                                            in.connectivity[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordSetOfFillAreaSets - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }

    status = PEXExtTMCoordTriangleStrip(&in.coord_data,
                                        in.vertex_fp_data_size,
                                        in.facet_att[obj],
                                        in.vertex_att[obj],
                                        in.color_type[obj],
                                        in.facet_array,
                                        in.POINTS[obj]-2,
                                        in.vertex_array[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordTriangleStrip - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }

    status = PEXExtTMCoordQuadrilateralMesh(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.facet_array,
                                            in.height[obj],
                                            in.width[obj],
                                            in.vertex_array[obj]);
    if(status != PEXExtBadValue)
    {
      fprintf(stderr,"Error: PEXExtTMCoordQuadrilateralMesh - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadValue\n");
    }
    in.coord_data.fp_data_index = int_sv;

 /*
  * Call with invalid normals
  * The err.obj included forces invalid normal calculations to occur.
  * The ploygon has four vertices that are equal.
  */
    fprintf(stderr, "Invalid Normals\n");

    status =
      PEXExtTMCoordFillAreaSetWithData(&in.coord_data, 
                                       in.vertex_fp_data_size,
                                       in.facet_att[obj], 
                                       in.vertex_att[obj], 
                                       in.color_type[obj],
                                       1, 
                                       in.facet_data, 
                                       areas);
    if(status != PEXExtBadPrimitive)
    {
      fprintf(stderr,"Error: PEXExtTMCoordFillAreaSetWithData - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadPrimitive\n");
    }
 
    status = PEXExtTMCoordSetOfFillAreaSets(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.count[obj],
                                            in.facet_array,
                                            in.POINTS[obj],
                                            in.vertex_array[obj],
                                            in.index_count[obj],
                                            in.connectivity[obj]);
    if(status != PEXExtBadPrimitive)
    {
      fprintf(stderr,"Error: PEXExtTMCoordSetOfFillAreaSets - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadPrimitive\n");
    }

    status = PEXExtTMCoordTriangleStrip(&in.coord_data,
                                        in.vertex_fp_data_size,
                                        in.facet_att[obj],
                                        in.vertex_att[obj],
                                        in.color_type[obj],
                                        in.facet_array,
                                        in.POINTS[obj]-2,
                                        in.vertex_array[obj]);
    if(status != PEXExtBadPrimitive)
    {
      fprintf(stderr,"Error: PEXExtTMCoordTriangleStrip - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadPrimitive\n");
    }

    status = PEXExtTMCoordQuadrilateralMesh(&in.coord_data,
                                            in.vertex_fp_data_size,
                                            in.facet_att[obj],
                                            in.vertex_att[obj],
                                            in.color_type[obj],
                                            in.facet_array,
                                            2,
                                            2,
                                            in.vertex_array[obj]);
    if(status != PEXExtBadPrimitive)
    {
      fprintf(stderr,"Error: PEXExtTMCoordQuadrilateralMesh - ");
      parse_error(status);
      fprintf (stderr, "Expected PEXExtBadPrimitive\n");
    }
} /* Test ends here */

 /**********************************************************************
  * Test ends here
  *********************************************************************/

 /*
  * Do a check on the test image.
  * Commented out for now because only stderr needs to be checked.
  */

    XSync(display, 0);

#ifndef NOT_XSYSTEM
    XSystemCheck("", &xtc);
#endif

#ifdef SUSPEX
    testproc(display, window, 1);
#endif

#ifdef BBA
    _bA_dump();
#endif 

#ifdef NOT_XSYSTEM
 /*
  * Close display
  */
{
char ch = getchar();
}
    XCloseDisplay(display);
#else
 /*
  * Terminate the testing system.
  */
    closeXSystemTesting(&xtc);
#endif
exit(0);
}

#ifdef SUSPEX
extern char *optarg;            /* for getopt */
extern int optind, opterr;

int parse_options2 (int argc, char **argv, int *pexmode, int *image_depth, 
    char *fname)
{
    int			c;
    char		dummy[2];

#ifndef XSYSTEM
    /* process command line options */
    while ((c = getopt(argc,argv,"d:inpsvwf:")) != EOF) {
        switch(c) {
            case 'd':
                *image_depth = atoi(optarg);
                break;
            case 'i':
                *pexmode |= IMM_MODE;
                break;
            case 'n':
                *pexmode |= NON_OC_MODE;
                break;
            case 'p':
                *pexmode |= WKS_MODE;
                break;
            case 's':
                *pexmode |= STR_MODE;
                break;
            case 'v':
                glob_verbose_mode = 1;
                break;
            case 'f':
                if(fname != NULL) {
		    strcpy(fname,optarg );
		}
                break;
	    case 'w':	/* wait for messing around with debugger :-) */
	        printf("Test waiting for <CR>: ");
	        scanf("%c", dummy);
		printf("-continuing\n");
	        break;
	    case '?':
	        printf("usage: %s: -[ipsvw]\n",argv[0]);
	        printf("-i		immediate mode\n");
	        printf("-n		non-oc mode\n");
	        printf("-p		PHIGS workstation mode\n");
	        printf("-s		structure rendering mode\n");
	        printf("-v		verbose mode (test descriptions)\n");
	        printf("-w		wait for <CR> before continuing\n");
	        printf("default	immediate mode\n");
	        return(1);
	}
    }
#endif

    return 0;
}
#endif
